@@ -34,8 +34,8 @@ def credentials():
 
     username = os.environ.get('OS_USERNAME')
     password = os.environ.get('OS_PASSWORD')
-    tenant_name = (os.environ.get('OS_TENANT_NAME')
-                   or os.environ.get('OS_PROJECT_NAME'))
+    tenant_name = (os.environ.get('OS_TENANT_NAME') or
+                   os.environ.get('OS_PROJECT_NAME'))
     auth_url = os.environ.get('OS_AUTH_URL')
 
     config = six.moves.configparser.RawConfigParser()
@@ -84,20 +84,6 @@ class ClientTestBase(base.ClientTestBase):
         for field in field_names:
             self.assertIn(field, headers)
 
-    def assertTableStruct(self, items, field_names):
-        """Verify that all items has keys listed in field_names.
-
-        :param items: items to assert are field names in the output table
-        :type items: list
-        :param field_names: field names from the output table of the cmd
-        :type field_names: list
-        """
-        # Strip off the --- if present
-
-        for item in items:
-            for field in field_names:
-                self.assertIn(field, item)
-
     def assert_object_details(self, expected, items):
         """Check presence of common object properties.
 
