@@ -56,6 +56,20 @@ def _stub_group_snapshot(detailed=True, **kwargs):
     return group_snapshot
 
 
+def _stub_snapshot(**kwargs):
+    snapshot = {
+        "created_at": "2012-08-28T16:30:31.000000",
+        "display_description": None,
+        "display_name": None,
+        "id": '11111111-1111-1111-1111-111111111111',
+        "size": 1,
+        "status": "available",
+        "volume_id": '00000000-0000-0000-0000-000000000000',
+    }
+    snapshot.update(kwargs)
+    return snapshot
+
+
 class FakeClient(fakes.FakeClient, client.Client):
 
     def __init__(self, api_version=None, *args, **kwargs):
@@ -63,8 +77,9 @@ class FakeClient(fakes.FakeClient, client.Client):
                                'project_id', 'auth_url',
                                extensions=kwargs.get('extensions'))
         self.api_version = api_version
+        global_id = "req-f551871a-4950-4225-9b2c-29a14c8f075e"
         self.client = FakeHTTPClient(api_version=api_version,
-                                     **kwargs)
+                global_request_id=global_id, **kwargs)
 
     def get_volume_api_version_from_endpoint(self):
         return self.client.get_volume_api_version_from_endpoint()
@@ -222,6 +237,44 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
         return (200, {},
                 {'backups': backup})
 
+    #
+    # Attachments
+    #
+    def post_attachments(self, **kw):
+        return (202, {}, {
+            'attachment': {'instance': 1234,
+                           'name': 'attachment-1',
+                           'volume_id': 'fake_volume_1',
+                           'status': 'reserved'}})
+
+    def get_attachments(self, **kw):
+        return (200, {}, {
+            'attachments': [{'instance': 1,
+                             'name': 'attachment-1',
+                             'volume_id': 'fake_volume_1',
+                             'status': 'reserved'},
+                            {'instance': 2,
+                             'name': 'attachment-2',
+                             'volume_id': 'fake_volume_2',
+                             'status': 'reserverd'}]})
+
+    def get_attachments_1234(self, **kw):
+        return (200, {}, {
+            'attachment': {'instance': 1234,
+                           'name': 'attachment-1',
+                           'volume_id': 'fake_volume_1',
+                           'status': 'reserved'}})
+
+    def put_attachments_1234(self, **kw):
+        return (200, {}, {
+            'attachment': {'instance': 1234,
+                           'name': 'attachment-1',
+                           'volume_id': 'fake_volume_1',
+                           'status': 'reserved'}})
+
+    def delete_attachments_1234(self, **kw):
+        return 204, {}, None
+
     #
     # GroupTypes
     #
@@ -317,6 +370,9 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
         action = list(body)[0]
         if action == 'delete':
             assert 'delete-volumes' in body[action]
+        elif action in ('enable_replication', 'disable_replication',
+                        'failover_replication', 'list_replication_targets'):
+            assert action in body
         else:
             raise AssertionError("Unexpected action: %s" % action)
         return (resp, {}, {})
@@ -361,6 +417,37 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
     def put_group_snapshots_1234(self, **kw):
         return (200, {}, {'group_snapshot': {}})
 
+    def post_groups_1234_action(self, **kw):
+        return (202, {}, {})
+
+    def get_groups_5678(self, **kw):
+        return (200, {}, {'group':
+                          _stub_group(id='5678')})
+
+    def post_groups_5678_action(self, **kw):
+        return (202, {}, {})
+
+    def post_snapshots_1234_action(self, **kw):
+        return (202, {}, {})
+
+    def get_snapshots_1234(self, **kw):
+        return (200, {}, {'snapshot': _stub_snapshot(id='1234')})
+
+    def post_snapshots_5678_action(self, **kw):
+        return (202, {}, {})
+
+    def get_snapshots_5678(self, **kw):
+        return (200, {}, {'snapshot': _stub_snapshot(id='5678')})
+
+    def post_group_snapshots_1234_action(self, **kw):
+        return (202, {}, {})
+
+    def post_group_snapshots_5678_action(self, **kw):
+        return (202, {}, {})
+
+    def get_group_snapshots_5678(self, **kw):
+        return (200, {}, {'group_snapshot': _stub_group_snapshot(id='5678')})
+
     def delete_group_snapshots_1234(self, **kw):
         return (202, {}, {})
 
@@ -461,6 +548,31 @@ class FakeHTTPClient(fake_v2.FakeHTTPClient):
         }
         return 200, {}, {'message': message}
 
+    def put_os_services_set_log(self, body):
+        return (202, {}, {})
+
+    def put_os_services_get_log(self, body):
+        levels = [{'binary': 'cinder-api', 'host': 'host1',
+                   'levels': {'prefix1': 'DEBUG', 'prefix2': 'INFO'}},
+                  {'binary': 'cinder-volume', 'host': 'host@backend#pool',
+                   'levels': {'prefix3': 'WARNING', 'prefix4': 'ERROR'}}]
+        return (200, {}, {'log_levels': levels})
+
+    def get_volumes_summary(self, **kw):
+        return 200, {}, {"volume-summary": {'total_size': 5,
+                                            'total_count': 5,
+                                            'metadata': {
+                                                "test_key": ["test_value"]
+                                            }
+                                            }
+                         }
+
+    #
+    # resource filters
+    #
+    def get_resource_filters(self, **kw):
+        return 200, {}, {'resource_filters': []}
+
 
 def fake_request_get():
     versions = {'versions': [{'id': 'v1.0',
@@ -500,3 +612,31 @@ def fake_request_get():
                               'updated': '2016-02-08T12:20:21Z',
                               'version': '3.16'}]}
     return versions
+
+
+def fake_request_get_no_v3():
+    versions = {'versions': [{'id': 'v1.0',
+                              'links': [{'href': 'http://docs.openstack.org/',
+                                         'rel': 'describedby',
+                                         'type': 'text/html'},
+                                        {'href': 'http://192.168.122.197/v1/',
+                                         'rel': 'self'}],
+                              'media-types': [{'base': 'application/json',
+                                               'type': 'application/'}],
+                              'min_version': '',
+                              'status': 'DEPRECATED',
+                              'updated': '2016-05-02T20:25:19Z',
+                              'version': ''},
+                             {'id': 'v2.0',
+                              'links': [{'href': 'http://docs.openstack.org/',
+                                         'rel': 'describedby',
+                                         'type': 'text/html'},
+                                        {'href': 'http://192.168.122.197/v2/',
+                                         'rel': 'self'}],
+                              'media-types': [{'base': 'application/json',
+                                               'type': 'application/'}],
+                              'min_version': '',
+                              'status': 'SUPPORTED',
+                              'updated': '2014-06-28T12:20:21Z',
+                              'version': ''}]}
+    return versions
